@@ -1,5 +1,4 @@
 use crate::Result;
-use duct::cmd;
 
 /// Generates shell completion scripts
 #[derive(Debug, clap::Args)]
@@ -12,14 +11,16 @@ pub struct Completion {
 
 impl Completion {
     pub async fn run(&self) -> Result<()> {
-        cmd!(
+        xx::process::cmd(
             "usage",
-            "g",
-            "completion",
-            &self.shell,
-            "hk",
-            "--usage-cmd",
-            "hk usage",
+            [
+                "g",
+                "completion",
+                &self.shell,
+                "hk",
+                "--usage-cmd",
+                "hk usage",
+            ],
         )
         .run()?;
         Ok(())
