@@ -63,6 +63,18 @@ impl Config {
         for (key, value) in self.env.iter() {
             unsafe { std::env::set_var(key, value) };
         }
+        // Set display_skip_reasons if configured (None means use default, empty list means hide all)
+        if let Some(display_skip_reasons) = &self.display_skip_reasons {
+            crate::settings::Settings::set_display_skip_reasons(
+                display_skip_reasons.clone().into_iter().collect(),
+            );
+        }
+        // Set hide_warnings if configured
+        if let Some(hide_warnings) = &self.hide_warnings {
+            crate::settings::Settings::set_hide_warnings(
+                hide_warnings.clone().into_iter().collect(),
+            );
+        }
         Ok(())
     }
 
@@ -132,6 +144,18 @@ impl Config {
                 crate::settings::Settings::set_check(check);
             }
 
+            if let Some(display_skip_reasons) = &user_config.display_skip_reasons {
+                crate::settings::Settings::set_display_skip_reasons(
+                    display_skip_reasons.clone().into_iter().collect(),
+                );
+            }
+
+            if let Some(hide_warnings) = &user_config.hide_warnings {
+                crate::settings::Settings::set_hide_warnings(
+                    hide_warnings.clone().into_iter().collect(),
+                );
+            }
+
             for (hook_name, user_hook_config) in &user_config.hooks {
                 if let Some(hook) = self.hooks.get_mut(hook_name) {
                     for (step_or_group_name, step_or_group) in hook.steps.iter_mut() {
@@ -234,6 +258,10 @@ pub struct Config {
     pub path: PathBuf,
     #[serde(default)]
     pub env: IndexMap<String, String>,
+    #[serde(rename = "display_skip_reasons")]
+    pub display_skip_reasons: Option<Vec<String>>,
+    #[serde(rename = "hide_warnings")]
+    pub hide_warnings: Option<Vec<String>>,
 }
 
 impl std::fmt::Display for Config {
@@ -258,6 +286,10 @@ pub struct UserConfig {
     pub defaults: UserDefaults,
     #[serde(default)]
     pub hooks: IndexMap<String, UserHookConfig>,
+    #[serde(rename = "display_skip_reasons")]
+    pub display_skip_reasons: Option<Vec<String>>,
+    #[serde(rename = "hide_warnings")]
+    pub hide_warnings: Option<Vec<String>>,
 }
 
 #[derive(Debug, Clone, Default, Deserialize, Serialize)]
