@@ -2,11 +2,12 @@ use clx::progress::{ProgressJob, ProgressJobBuilder, ProgressOutput, ProgressSta
 use indexmap::IndexMap;
 use itertools::Itertools;
 use serde::{Deserialize, Serialize};
+use serde_with::{DisplayFromStr, PickFirst, serde_as};
 use std::{
     collections::{BTreeSet, HashSet},
     ffi::OsString,
     path::{Path, PathBuf},
-    sync::Arc,
+    sync::{Arc, Mutex as StdMutex},
 };
 use tokio::{
     signal,
@@ -21,7 +22,7 @@ use crate::{
     glob,
     hook_options::HookOptions,
     settings::Settings,
-    step::{CheckType, EXPR_CTX, RunType, Step},
+    step::{CheckType, EXPR_CTX, RunType, Script, Step},
     step_context::StepContext,
     step_group::{StepGroup, StepGroupContext},
     timings::TimingRecorder,
@@ -29,6 +30,54 @@ use crate::{
     version,
 };
 
+#[derive(Debug, Clone, Eq, PartialEq, strum::Display)]
+#[strum(serialize_all = "kebab-case")]
+pub enum SkipReason {
+    #[strum(serialize = "disabled-by-env")]
+    DisabledByEnv(String),
+    #[strum(serialize = "disabled-by-cli")]
+    DisabledByCli(String),
+    ProfileNotEnabled(Vec<String>),
+    ProfileExplicitlyDisabled,
+    #[strum(serialize = "no-command-for-run-type")]
+    NoCommandForRunType(RunType),
+    NoFilesToProcess,
+    ConditionFalse,
+}
+
+impl SkipReason {
+    pub fn message(&self) -> String {
+        match self {
+            SkipReason::DisabledByEnv(src) | SkipReason::DisabledByCli(src) => {
+                format!("skipped: disabled via {src}")
+            }
+            SkipReason::ProfileNotEnabled(profiles) => {
+                if profiles.is_empty() {
+                    "skipped: disabled by profile".to_string()
+                } else {
+                    format!(
+                        "skipped: missing profile{} ({})",
+                        if profiles.len() > 1 { "s" } else { "" },
+                        profiles.join(", ")
+                    )
+                }
+            }
+            SkipReason::ProfileExplicitlyDisabled => "skipped: disabled by profile".to_string(),
+            SkipReason::NoCommandForRunType(_) => "skipped: no command for run type".to_string(),
+            SkipReason::NoFilesToProcess => "skipped: no files to process".to_string(),
+            SkipReason::ConditionFalse => "skipped: condition is false".to_string(),
+        }
+    }
+
+    pub fn should_display(&self) -> bool {
+        let settings = Settings::get();
+        // Use strum's Display trait to get the kebab-case string
+        let key = self.to_string();
+        settings.display_skip_reasons.contains(&key)
+    }
+}
+
+#[serde_as]
 #[derive(Debug, Clone, Default, Deserialize, Serialize, Eq, PartialEq)]
 #[serde(rename_all = "snake_case")]
 #[cfg_attr(debug_assertions, serde(deny_unknown_fields))]
@@ -39,6 +88,8 @@ pub struct Hook {
     pub steps: IndexMap<String, StepOrGroup>,
     pub fix: Option<bool>,
     pub stash: Option<StashMethod>,
+    #[serde_as(as = "Option<PickFirst<(_, DisplayFromStr)>>")]
+    pub report: Option<Script>,
 }
 
 #[derive(Debug, Clone, Deserialize, Serialize, Eq, PartialEq)]
@@ -70,7 +121,9 @@ pub struct HookContext {
     total_jobs: std::sync::Mutex<usize>,
     completed_jobs: std::sync::Mutex<usize>,
     expr_ctx: std::sync::Mutex<expr::Context>,
-    pub timing: Option<Arc<TimingRecorder>>,
+    pub timing: Arc<TimingRecorder>,
+    pub skip_steps: IndexMap<String, SkipReason>,
+    skipped_steps: std::sync::Mutex<IndexMap<String, SkipReason>>,
 }
 
 impl HookContext {
@@ -81,27 +134,35 @@ impl HookContext {
         tctx: crate::tera::Context,
         run_type: RunType,
         hk_progress: Option<Arc<ProgressJob>>,
+        skip_steps: IndexMap<String, SkipReason>,
     ) -> Self {
         let settings = Settings::get();
         let expr_ctx = EXPR_CTX.clone();
-        let timing = env::HK_TIMING_JSON
-            .as_ref()
-            .map(|path| Arc::new(TimingRecorder::new(path.clone())));
+        let mut timing = TimingRecorder::new(env::HK_TIMING_JSON.clone());
+        // Pre-populate timing metadata once before any jobs start
+        for group in &groups {
+            for step in group.steps.values() {
+                timing.set_step_profiles(&step.name, step.profiles.as_deref());
+                timing.set_step_interactive(&step.name, step.interactive);
+            }
+        }
         Self {
             file_locks: FileRwLocks::new(files),
             git,
             hk_progress,
-            total_jobs: std::sync::Mutex::new(groups.iter().map(|g| g.steps.len()).sum()),
-            completed_jobs: std::sync::Mutex::new(0),
+            total_jobs: StdMutex::new(groups.iter().map(|g| g.steps.len()).sum()),
+            completed_jobs: StdMutex::new(0),
             groups,
             tctx,
             run_type,
-            step_contexts: std::sync::Mutex::new(Default::default()),
-            files_in_contention: std::sync::Mutex::new(Default::default()),
+            step_contexts: StdMutex::new(Default::default()),
+            files_in_contention: StdMutex::new(Default::default()),
             semaphore: Arc::new(Semaphore::new(settings.jobs.get())),
             failed: CancellationToken::new(),
-            expr_ctx: std::sync::Mutex::new(expr_ctx),
-            timing,
+            expr_ctx: StdMutex::new(expr_ctx),
+            timing: Arc::new(timing),
+            skip_steps,
+            skipped_steps: StdMutex::new(IndexMap::new()),
         }
     }
 
@@ -154,6 +215,17 @@ impl HookContext {
             }
         }
     }
+
+    pub fn track_skip(&self, step_name: &str, reason: SkipReason) {
+        self.skipped_steps
+            .lock()
+            .unwrap()
+            .insert(step_name.to_string(), reason);
+    }
+
+    pub fn get_skipped_steps(&self) -> IndexMap<String, SkipReason> {
+        self.skipped_steps.lock().unwrap().clone()
+    }
 }
 
 impl Hook {
@@ -173,7 +245,7 @@ impl Hook {
         }
     }
 
-    fn get_step_groups(&self, run_type: RunType, opts: &HookOptions) -> Vec<StepGroup> {
+    fn get_step_groups(&self, opts: &HookOptions) -> Vec<StepGroup> {
         let mut steps = self.steps.values().cloned().collect_vec();
         if !opts.step.is_empty() {
             steps = steps
@@ -187,33 +259,12 @@ impl Hook {
                 })
                 .collect_vec();
         }
-        let step_ok = |step: &Step| {
-            if step.run_cmd(run_type).is_none() {
-                debug!("{step}: skipping step due to no available run type");
-                false
-            } else if env::HK_SKIP_STEPS.contains(&step.name) {
-                debug!("{step}: skipping step due to HK_SKIP_STEPS");
-                false
-            } else {
-                step.is_profile_enabled()
-            }
-        };
-        let steps = steps
-            .into_iter()
-            .filter_map(|s| match s {
-                StepOrGroup::Step(ref step) => step_ok(step).then_some(s),
-                StepOrGroup::Group(mut group) => {
-                    group.steps.retain(|_, s| step_ok(s));
-                    Some(StepOrGroup::Group(group))
-                }
-            })
-            .collect_vec();
         StepGroup::build_all(steps)
     }
 
     pub async fn plan(&self, opts: HookOptions) -> Result<()> {
         let run_type = self.run_type(&opts);
-        let groups = self.get_step_groups(run_type, &opts);
+        let groups = self.get_step_groups(&opts);
         let repo = Arc::new(Mutex::new(Git::new()?));
         let git_status = OnceCell::new();
         let stash_method = env::HK_STASH.or(self.stash).unwrap_or(StashMethod::None);
@@ -223,7 +274,7 @@ impl Hook {
         let files = self
             .file_list(&opts, repo.clone(), &git_status, stash_method, &progress)
             .await?;
-        if files.is_empty() && can_exit_early(&groups, &files, run_type) {
+        if files.is_empty() && can_exit_early(&groups, &files, run_type, &IndexMap::new()) {
             info!("no files to run");
             return Ok(());
         }
@@ -245,9 +296,10 @@ impl Hook {
         let run_type = self.run_type(&opts);
         let repo = Arc::new(Mutex::new(Git::new()?));
         let git_status = OnceCell::new();
-        let groups = self.get_step_groups(run_type, &opts);
+        let groups = self.get_step_groups(&opts);
         let stash_method = env::HK_STASH.or(self.stash).unwrap_or(StashMethod::None);
-        let hk_progress = self.start_hk_progress(run_type, groups.len());
+        let total_steps: usize = groups.iter().map(|g| g.steps.len()).sum();
+        let hk_progress = self.start_hk_progress(run_type, total_steps);
         let file_progress = ProgressJobBuilder::new().body(
             "{{spinner()}} files - {{message}}{% if files is defined %} ({{files}} file{{files|pluralize}}){% endif %}"
         )
@@ -263,7 +315,23 @@ impl Hook {
             )
             .await?;
 
-        if files.is_empty() && can_exit_early(&groups, &files, run_type) {
+        let skip_steps = {
+            let mut m: IndexMap<String, SkipReason> = IndexMap::new();
+            for s in env::HK_SKIP_STEPS.iter() {
+                m.insert(
+                    s.clone(),
+                    SkipReason::DisabledByEnv("HK_SKIP_STEPS".to_string()),
+                );
+            }
+            for s in opts.skip_step.iter() {
+                m.insert(
+                    s.clone(),
+                    SkipReason::DisabledByCli(format!("--skip-step {}", s)),
+                );
+            }
+            m
+        };
+        if files.is_empty() && can_exit_early(&groups, &files, run_type, &skip_steps) {
             info!("no files to run");
             if let Some(hk_progress) = &hk_progress {
                 hk_progress.set_status(ProgressStatus::Hide);
@@ -277,6 +345,7 @@ impl Hook {
             opts.tctx,
             run_type,
             hk_progress,
+            skip_steps,
         ));
 
         watch_for_ctrl_c(hook_ctx.failed.clone());
@@ -324,9 +393,84 @@ impl Hook {
                 warn!("Failed to pop stash: {err}");
             }
         }
-        if let Some(timing) = &hook_ctx.timing {
-            if let Err(err) = timing.write_json() {
-                warn!("Failed to write timing JSON: {err}");
+        if let Err(err) = hook_ctx.timing.write_json() {
+            warn!("Failed to write timing JSON: {err}");
+        }
+
+        // Clear progress bars before displaying summary
+        clx::progress::stop();
+
+        // Display summary of profile-skipped steps
+        // Only show summary if profile-not-enabled is configured to be displayed
+        if Settings::get()
+            .display_skip_reasons
+            .contains("profile-not-enabled")
+        {
+            let skipped_steps = hook_ctx.get_skipped_steps();
+            let mut profile_skipped: Vec<String> = vec![];
+            let mut missing_profiles = indexmap::IndexSet::new();
+
+            for (name, reason) in skipped_steps.iter() {
+                if let SkipReason::ProfileNotEnabled(profiles) = reason {
+                    profile_skipped.push(name.clone());
+                    missing_profiles.extend(profiles.clone());
+                }
+            }
+
+            if !profile_skipped.is_empty() {
+                let count = profile_skipped.len();
+                let profiles_list = missing_profiles.iter().join(", ");
+                tagged_warn_missing_profiles!(
+                    "{count} {} skipped due to missing profiles: {profiles_list}",
+                    if count == 1 { "step was" } else { "steps were" },
+                );
+
+                // Show appropriate help message based on hook type
+                let (hk_profile_env, hk_profile_flag) = if missing_profiles.contains("slow") {
+                    ("HK_PROFILE=slow".to_string(), "--slow".to_string())
+                } else {
+                    let default = "slow".to_string();
+                    let profile = missing_profiles.iter().next().unwrap_or(&default);
+                    (
+                        format!("HK_PROFILE={profile}"),
+                        format!("--profile={profile}"),
+                    )
+                };
+                let hk_profile_env = style::edim(hk_profile_env);
+                let hk_profile_flag = style::edim(hk_profile_flag);
+                if self.name == "pre-commit" || self.name == "pre-push" {
+                    tagged_warn_missing_profiles!(
+                        "   To enable these steps, set {hk_profile_env} environment variable."
+                    );
+                } else {
+                    tagged_warn_missing_profiles!(
+                        "   To enable these steps, use {hk_profile_flag} or set {hk_profile_env}."
+                    );
+                }
+                let hide_warning_env = style::edim("HK_HIDE_WARNINGS=missing-profiles");
+                tagged_warn_missing_profiles!("   To hide this warning: set {hide_warning_env}");
+                let hide_warning_pkl = style::edim(r#"hide_warnings = List("missing-profiles")"#);
+                tagged_warn_missing_profiles!("   or set {hide_warning_pkl} in .hkrc.pkl");
+            }
+        }
+
+        // Run hook-level report if configured
+        if let Some(report) = &self.report {
+            if let Ok(json) = hook_ctx.timing.to_json_string() {
+                let mut cmd = ensembler::CmdLineRunner::new("sh")
+                    .arg("-o")
+                    .arg("errexit")
+                    .arg("-c");
+                let run = report.to_string();
+                cmd = cmd.arg(&run).env("HK_REPORT_JSON", json);
+                let pr = ProgressJobBuilder::new()
+                    .body("report: {{message}}")
+                    .prop("message", &run)
+                    .start();
+                cmd = cmd.with_pr(pr);
+                if let Err(err) = cmd.execute().await {
+                    warn!("Report command failed: {err}");
+                }
             }
         }
         result
@@ -493,12 +637,18 @@ fn all_files_in_dir(dir: &Path) -> Result<Vec<PathBuf>> {
     Ok(files)
 }
 
-fn can_exit_early(groups: &[StepGroup], files: &BTreeSet<PathBuf>, run_type: RunType) -> bool {
+fn can_exit_early(
+    groups: &[StepGroup],
+    files: &BTreeSet<PathBuf>,
+    run_type: RunType,
+    skip_steps: &IndexMap<String, SkipReason>,
+) -> bool {
     let files = files.iter().cloned().collect::<Vec<_>>();
     groups.iter().all(|g| {
         g.steps.iter().all(|(_, s)| {
-            s.build_step_jobs(&files, run_type, &Default::default())
-                .is_ok_and(|jobs| jobs.is_empty())
+            // Reuse job builder to determine if this step has any runnable work
+            s.build_step_jobs(&files, run_type, &Default::default(), skip_steps)
+                .is_ok_and(|jobs| jobs.iter().all(|j| j.skip_reason.is_some()))
         })
     })
 }
