@@ -1,8 +1,8 @@
-use crate::timings::StepTimingGuard;
 use crate::{Result, error::Error, step_job::StepJob};
 use crate::{env, step_job::StepJobStatus};
 use crate::{glob, settings::Settings};
-use crate::{step_context::StepContext, tera};
+use crate::{hook::SkipReason, timings::StepTimingGuard};
+use crate::{step_context::StepContext, tera, ui::style};
 use clx::progress::{ProgressJob, ProgressJobBuilder, ProgressJobDoneBehavior, ProgressStatus};
 use ensembler::CmdLineRunner;
 use eyre::{WrapErr, eyre};
@@ -150,12 +150,36 @@ impl Step {
         })
     }
 
-    pub fn is_profile_enabled(&self) -> bool {
-        is_profile_enabled(
-            &self.name,
-            self.enabled_profiles(),
-            self.disabled_profiles(),
-        )
+    pub fn profile_skip_reason(&self) -> Option<SkipReason> {
+        let settings = Settings::get();
+        if let Some(enabled) = self.enabled_profiles() {
+            let missing_profiles = enabled
+                .difference(&settings.enabled_profiles)
+                .collect::<Vec<_>>();
+            if !missing_profiles.is_empty() {
+                let profiles = missing_profiles
+                    .into_iter()
+                    .map(|s| s.to_string())
+                    .collect();
+                return Some(SkipReason::ProfileNotEnabled(profiles));
+            }
+            let disabled_profiles = settings
+                .disabled_profiles
+                .intersection(&enabled)
+                .collect_vec();
+            if !disabled_profiles.is_empty() {
+                return Some(SkipReason::ProfileExplicitlyDisabled);
+            }
+        }
+        if let Some(disabled) = self.disabled_profiles() {
+            let disabled_profiles = disabled
+                .intersection(&settings.enabled_profiles)
+                .collect::<Vec<_>>();
+            if !disabled_profiles.is_empty() {
+                return Some(SkipReason::ProfileExplicitlyDisabled);
+            }
+        }
+        None
     }
 
     pub(crate) fn build_step_progress(&self) -> Arc<ProgressJob> {
@@ -243,12 +267,32 @@ impl Step {
         files: &[PathBuf],
         run_type: RunType,
         files_in_contention: &HashSet<PathBuf>,
+        skip_steps: &indexmap::IndexMap<String, crate::hook::SkipReason>,
     ) -> Result<Vec<StepJob>> {
+        // Pre-calculate skip reason at the job creation level to simplify run_all_jobs
+        if skip_steps.contains_key(&self.name) {
+            let reason = skip_steps.get(&self.name).unwrap().clone();
+            let mut j = StepJob::new(Arc::new(self.clone()), vec![], run_type);
+            j.skip_reason = Some(reason);
+            return Ok(vec![j]);
+        }
+        if let Some(reason) = self.profile_skip_reason() {
+            let mut j = StepJob::new(Arc::new(self.clone()), vec![], run_type);
+            j.skip_reason = Some(reason);
+            return Ok(vec![j]);
+        }
+        if self.run_cmd(run_type).is_none() {
+            let mut j = StepJob::new(Arc::new(self.clone()), vec![], run_type);
+            j.skip_reason = Some(SkipReason::NoCommandForRunType(run_type));
+            return Ok(vec![j]);
+        }
         let files = self.filter_files(files)?;
         if files.is_empty() && (self.glob.is_some() || self.dir.is_some() || self.exclude.is_some())
         {
             debug!("{self}: no file matches for step");
-            return Ok(Default::default());
+            let mut j = StepJob::new(Arc::new(self.clone()), vec![], run_type);
+            j.skip_reason = Some(SkipReason::NoFilesToProcess);
+            return Ok(vec![j]);
         }
         let mut jobs = if let Some(workspace_indicators) = self.workspaces_for_files(&files)? {
             let mut files = files.clone();
@@ -309,22 +353,30 @@ impl Step {
             &files,
             ctx.hook_ctx.run_type,
             &ctx.hook_ctx.files_in_contention.lock().unwrap(),
+            &ctx.hook_ctx.skip_steps,
         )?;
         if let Some(job) = jobs.first_mut() {
             job.semaphore = Some(semaphore);
-        } else {
-            ctx.depends.mark_done(&self.name)?;
-            debug!("{self}: no jobs to run");
-            return Ok(());
         }
-        ctx.set_jobs_total(jobs.len());
+        let non_skip_jobs = jobs.iter().filter(|j| j.skip_reason.is_none()).count();
+        ctx.set_jobs_total(non_skip_jobs);
+        if non_skip_jobs > 0 {
+            // Replace the single-step placeholder with the actual number of jobs
+            // Add the extra jobs beyond the placeholder 1
+            ctx.hook_ctx.inc_total_jobs(non_skip_jobs.saturating_sub(1));
+        }
         let mut set = tokio::task::JoinSet::new();
         for job in jobs {
             let ctx = ctx.clone();
             let step = self.clone();
             let mut job = job;
             set.spawn(async move {
-                ctx.hook_ctx.inc_total_jobs(1);
+                if let Some(reason) = &job.skip_reason {
+                    step.mark_skipped(&ctx, reason)?;
+                    // Skipped jobs should still count as completed for overall progress
+                    ctx.hook_ctx.inc_completed_jobs(1);
+                    return Ok(());
+                }
                 if job.check_first {
                     let prev_run_type = job.run_type;
                     job.run_type = RunType::Check(step.check_type());
@@ -332,6 +384,7 @@ impl Step {
                     match step.run(&ctx, &mut job).await {
                         Ok(()) => {
                             debug!("{step}: successfully ran check step first");
+                            ctx.hook_ctx.inc_completed_jobs(1);
                             return Ok(());
                         }
                         Err(e) => {
@@ -359,26 +412,16 @@ impl Step {
                 if let Err(err) = &result {
                     job.status_errored(&ctx, format!("{err}")).await?;
                 }
+                ctx.hook_ctx.inc_completed_jobs(1);
                 result
             });
         }
         while let Some(res) = set.join_next().await {
-            ctx.hook_ctx.inc_completed_jobs(1);
             match res {
                 Ok(Ok(())) => {}
                 Ok(Err(err)) => {
                     ctx.status_errored(&format!("{err}"));
                     return Err(err);
-                    // TODO: abort all jobs after a timeout
-                    // tokio::spawn(async move {
-                    //     tokio::time::sleep(Duration::from_secs(5)).await;
-                    //     set.abort_all();
-                    // });
-                    // if child.is_running() {
-                    //     child.set_status(clx::progress::ProgressStatus::DoneCustom(
-                    //         style::eyellow("▲").to_string(),
-                    //     ));
-                    // }
                 }
                 Err(e) => match e.try_into_panic() {
                     Ok(e) => std::panic::resume_unwind(e),
@@ -393,7 +436,7 @@ impl Step {
             ctx.status_aborted();
             return Ok(());
         }
-        if matches!(ctx.hook_ctx.run_type, RunType::Fix) {
+        if non_skip_jobs > 0 && matches!(ctx.hook_ctx.run_type, RunType::Fix) {
             // Build stage pathspecs; if `dir` is set, stage entries are relative to it
             let stage: Vec<OsString> = self
                 .stage
@@ -419,8 +462,10 @@ impl Step {
                 }
             }
         }
-        ctx.status_finished();
-        ctx.depends.mark_done(&self.name)?;
+        if non_skip_jobs > 0 {
+            ctx.status_finished();
+            ctx.depends.mark_done(&self.name)?;
+        }
         Ok(())
     }
 
@@ -445,12 +490,17 @@ impl Step {
     pub(crate) async fn run(&self, ctx: &StepContext, job: &mut StepJob) -> Result<()> {
         if ctx.hook_ctx.failed.is_cancelled() {
             trace!("{self}: skipping step due to previous failure");
+            // Hide the job progress if it was created
+            if let Some(progress) = &job.progress {
+                progress.set_status(ProgressStatus::Hide);
+            }
             return Ok(());
         }
         if let Some(condition) = &self.condition {
             let val = EXPR_ENV.eval(condition, &ctx.hook_ctx.expr_ctx())?;
-            trace!("{self}: condition: {condition} = {val}");
+            debug!("{self}: condition: {condition} = {val}");
             if val == expr::Value::Bool(false) {
+                self.mark_skipped(ctx, &SkipReason::ConditionFalse)?;
                 return Ok(());
             }
         }
@@ -508,7 +558,8 @@ impl Step {
             .arg(&run)
             .with_pr(job.progress.as_ref().unwrap().clone())
             .with_cancel_token(ctx.hook_ctx.failed.clone())
-            .show_stderr_on_error(false);
+            .show_stderr_on_error(false)
+            .stderr_to_progress(true);
         if self.interactive {
             clx::progress::pause();
             cmd = cmd
@@ -523,15 +574,9 @@ impl Step {
             let value = tera::render(value, &tctx)?;
             cmd = cmd.env(key, value);
         }
-        let mut timing_guard = ctx
-            .hook_ctx
-            .timing
-            .clone()
-            .map(|t| StepTimingGuard::new(t, self));
+        let timing_guard = StepTimingGuard::new(ctx.hook_ctx.timing.clone(), self);
         let exec_result = cmd.execute().await;
-        if let Some(g) = timing_guard.take() {
-            g.finish();
-        }
+        timing_guard.finish();
         if self.interactive {
             clx::progress::resume();
         }
@@ -578,6 +623,22 @@ impl Step {
             _ => ShellType::Other(shell.to_string()),
         }
     }
+
+    pub fn mark_skipped(&self, ctx: &StepContext, reason: &SkipReason) -> Result<()> {
+        // Track all skip reasons for potential future use
+        ctx.hook_ctx.track_skip(&self.name, reason.clone());
+
+        if reason.should_display() {
+            ctx.progress.prop("message", &reason.message());
+            let status = ProgressStatus::DoneCustom(style::eblue("⏭").bold().to_string());
+            ctx.progress.set_status(status);
+        } else {
+            // Step is skipped but message shouldn't be displayed
+            ctx.progress.set_status(ProgressStatus::Hide);
+        }
+        ctx.depends.mark_done(&self.name)?;
+        Ok(())
+    }
 }
 
 pub enum ShellType {
@@ -603,43 +664,6 @@ impl ShellType {
         }
     }
 }
-fn is_profile_enabled(
-    name: &str,
-    enabled: Option<IndexSet<String>>,
-    disabled: Option<IndexSet<String>>,
-) -> bool {
-    let settings = Settings::get();
-    if let Some(enabled) = enabled {
-        let missing_profiles = enabled
-            .difference(&settings.enabled_profiles)
-            .collect::<Vec<_>>();
-        if !missing_profiles.is_empty() {
-            let missing_profiles = missing_profiles.iter().join(", ");
-            debug!("{name}: skipping step due to missing profile: {missing_profiles}");
-            return false;
-        }
-        let disabled_profiles = settings
-            .disabled_profiles
-            .intersection(&enabled)
-            .collect_vec();
-        if !disabled_profiles.is_empty() {
-            let disabled_profiles = disabled_profiles.iter().join(", ");
-            debug!("{name}: skipping step due to disabled profile: {disabled_profiles}");
-            return false;
-        }
-    }
-    if let Some(disabled) = disabled {
-        let disabled_profiles = disabled
-            .intersection(&settings.enabled_profiles)
-            .collect::<Vec<_>>();
-        if !disabled_profiles.is_empty() {
-            let disabled_profiles = disabled_profiles.iter().join(", ");
-            debug!("{name}: skipping step due to disabled profile: {disabled_profiles}");
-            return false;
-        }
-    }
-    true
-}
 
 pub static EXPR_CTX: LazyLock<expr::Context> = LazyLock::new(expr::Context::default);
 
