@@ -1,6 +1,6 @@
 use clx::progress::{ProgressJob, ProgressJobBuilder, ProgressStatus};
 use eyre::Context;
-use indexmap::IndexMap;
+use indexmap::{IndexMap, IndexSet};
 use serde::{Deserialize, Serialize};
 
 use crate::{
@@ -12,7 +12,7 @@ use crate::{hook::HookContext, step::Step};
 use std::{
     collections::{HashMap, HashSet},
     path::{Path, PathBuf},
-    sync::Arc,
+    sync::{Arc, Mutex},
 };
 
 #[derive(Debug, Clone, Default, Deserialize, Serialize, Eq, PartialEq)]
@@ -97,6 +97,7 @@ impl StepGroup {
 
     pub async fn run(&self, ctx: StepGroupContext) -> Result<()> {
         let settings = Settings::get();
+        // timing metadata already pre-populated in HookContext::new
         let depends = Arc::new(StepDepends::new(
             &self
                 .steps
@@ -116,9 +117,9 @@ impl StepGroup {
                         hook_ctx: ctx.hook_ctx.clone(),
                         depends: depends.clone(),
                         progress: s.build_step_progress(),
-                        files_added: Arc::new(std::sync::Mutex::new(0)),
-                        jobs_remaining: Arc::new(std::sync::Mutex::new(0)),
-                        jobs_total: std::sync::Mutex::new(0),
+                        files_added: Arc::new(Mutex::new(IndexSet::new())),
+                        jobs_remaining: Arc::new(Mutex::new(0)),
+                        jobs_total: Mutex::new(0),
                         status: Default::default(),
                     }),
                 )
@@ -158,12 +159,15 @@ impl StepGroup {
         }
         let mut result = Ok(());
         while let Some(res) = set.join_next().await {
-            ctx.hook_ctx.inc_completed_jobs(1);
             match res {
                 Ok(Ok(())) => {}
                 Ok(Err(err)) => {
                     if settings.fail_fast {
                         ctx.hook_ctx.failed.cancel();
+                        // Mark remaining steps as aborted
+                        for step_ctx in ctx.hook_ctx.step_contexts.lock().unwrap().values() {
+                            step_ctx.status_aborted();
+                        }
                         return Err(err);
                     } else if result.is_ok() {
                         result = Err(err);
