@@ -1,4 +1,4 @@
-use crate::{Result, file_rw_locks::Flocks};
+use crate::{Result, file_rw_locks::Flocks, hook::SkipReason};
 use clx::progress::{ProgressJob, ProgressJobBuilder, ProgressJobDoneBehavior, ProgressStatus};
 use itertools::Itertools;
 use tokio::sync::OwnedSemaphorePermit;
@@ -20,6 +20,7 @@ pub struct StepJob {
     pub files: Vec<PathBuf>,
     pub run_type: RunType,
     pub check_first: bool,
+    pub skip_reason: Option<SkipReason>,
     pub progress: Option<Arc<ProgressJob>>,
     pub semaphore: Option<OwnedSemaphorePermit>,
     workspace_indicator: Option<PathBuf>,
@@ -50,6 +51,7 @@ impl StepJob {
                 && matches!(run_type, RunType::Fix),
             step,
             status: StepJobStatus::Pending,
+            skip_reason: None,
             progress: None,
             semaphore: None,
         }
@@ -85,11 +87,10 @@ impl StepJob {
             .prop("name", &self.step.name)
             .prop("files", &self.files.iter().map(|f| f.display()).join(" "))
             .body(
-                // TODO: truncate properly
-                "{{spinner()}} {% if ensembler_cmd %}{{ensembler_cmd | flex}}\n{{ensembler_stdout | flex}}{% else %}{{message | flex}}{% endif %}"
+                "{{spinner()}} {% if ensembler_cmd %}{{ensembler_cmd | flex}}{% if ensembler_stdout %}\n{{ensembler_stdout | flex}}{% endif %}{% else %}{{message | flex}}{% endif %}"
             )
             .body_text(Some(
-                "{% if ensembler_stdout %}  {{name}} – {{ensembler_stdout}}{% elif message %}{{spinner()}} {{name}} – {{message}}{% endif %}".to_string(),
+                "{% if ensembler_stdout %}  {{name}} – {{ensembler_stdout | truncate_text}}{% elif message %}{{spinner()}} {{name}} – {{message | truncate_text}}{% endif %}".to_string(),
             ))
             .on_done(ProgressJobDoneBehavior::Hide)
             .build();
@@ -161,6 +162,7 @@ impl Clone for StepJob {
             files: self.files.clone(),
             run_type: self.run_type,
             check_first: self.check_first,
+            skip_reason: self.skip_reason.clone(),
             workspace_indicator: self.workspace_indicator.clone(),
             status: StepJobStatus::Pending,
             progress: self.progress.clone(),
